import site
from typing import Any


def pytest_configure(config: Any) -> None:
    return
    # See: https://github.com/typeddjango/pytest-mypy-plugins/issues/34
    site_packages = site.getsitepackages()[0]
    for file_name, line_numbers in FILES:
        file_path = f"{site_packages}/{file_name}.pyi"

        with open(file_path) as file_in:
            file_lines = []

            for index, line in enumerate(file_in.readlines()):
                line_number = index + 1
                if line_number in line_numbers and "ignore" not in line:
                    file_lines.append(
                        f"{line.rstrip()}  # type: ignore[override,var-annotated,import]\n"
                    )
                else:
                    file_lines.append(line)
        with open(file_path, "w") as file_out:
            file_out.writelines(file_lines)


FILES = [
    ("django-stubs/core/files/base", (24, 37)),
    ("django-stubs/contrib/auth/base_user", (18, 19, 25, 26)),
    (
        "django-stubs/contrib/auth/models",
        (
            23,
            24,
            25,
            29,
            30,
            31,
            34,
            5,
            40,
            41,
            57,
            58,
            59,
            63,
            64,
            65,
            70,
            71,
            72,
            73,
            74,
            75,
            76,
            77,
            78,
            79,
            80,
            81,
            82,
        ),
    ),
    ("django-stubs/contrib/postgres/fields/ranges", (4,)),
    ("django-stubs/contrib/sites/models", (16, 17, 18)),
]



import json

from django.test import RequestFactory

from reactivated.renderer import get_accept_list, render_jsx_to_string


def test_get_accept_list():
    request_factory = RequestFactory(
        headers={
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3"
        }
    )
    request = request_factory.get("/")
    accepts = get_accept_list(request)
    assert accepts == [
        "text/html",
        "application/xhtml+xml",
        "application/xml;q=0.9",
        "image/webp",
        "image/apng",
        "*/*;q=0.8",
        "application/signed-exchange;v=b3",
    ]


def test_render_to_json():
    request_factory = RequestFactory(headers={"accept": "application/json"})
    request = request_factory.get("/")
    response = render_jsx_to_string(
        request,
        {"template_name": "doesnotmatter.tsx", "some": "property"},
        {"another": "property"},
    )
    assert json.loads(response) == {
        "context": {"some": "property", "template_name": "doesnotmatter.tsx"},
        "props": {"another": "property"},
    }



import enum
from typing import NamedTuple

from django import forms

from reactivated import export, serialization
from reactivated.apps import get_values
from reactivated.serialization.registry import value_registry


class SimpleEnum(enum.Enum):
    FIRST = "First"
    SECOND = "Second"


class ComplexMember(NamedTuple):
    ranking: int


class ComplexEnum(enum.Enum):
    FIRST = ComplexMember(ranking=1)
    SECOND = ComplexMember(ranking=2)


class SimpleForm(forms.Form):
    some_field = forms.CharField()


def test_export_registry():
    FOO = 1
    BAR = 2

    export(FOO)

    export(BAR)

    assert value_registry["FOO"] == (1, "primitive")
    assert value_registry["BAR"] == (2, "primitive")


def test_export_complex_types():
    value_registry["SimpleForm"] = [SimpleForm, False]
    generated_schema = serialization.create_schema(SimpleForm, {})
    assert get_values()["SimpleForm"] == serialization.serialize(
        SimpleForm, generated_schema
    )

    del value_registry["SimpleForm"]


def test_export_enum():
    export(SimpleEnum)
    # generated_schema = serialization.create_schema(Type[SimpleEnum], {})
    assert get_values()["tests.exports.SimpleEnum"] == {
        "FIRST": "First",
        "SECOND": "Second",
    }

    export(ComplexEnum)
    assert get_values()["tests.exports.ComplexEnum"] == {
        "FIRST": {"ranking": 1},
        "SECOND": {"ranking": 2},
    }


def test_decorator_export():
    @export()
    class DecoratedEnum(enum.Enum):
        FIRST = "First"
        SECOND = "Second"

    assert get_values()[
        "tests.exports.test_decorator_export.<locals>.DecoratedEnum"
    ] == {
        "FIRST": "First",
        "SECOND": "Second",
    }



from . import registry


@registry.register(int)
class Int:
    @classmethod
    def get_json_schema(
        Proxy: type["Int"],
        instance: int,
        definitions: registry.Definitions,
    ) -> registry.Thing:
        return registry.Thing(schema={"type": "number"}, definitions=definitions)


@registry.register(bool)
class Bool:
    @classmethod
    def get_json_schema(
        Proxy: type["Bool"],
        instance: bool,
        definitions: registry.Definitions,
    ) -> registry.Thing:
        return registry.Thing(schema={"type": "boolean"}, definitions=definitions)


@registry.register(float)
class Float:
    @classmethod
    def get_json_schema(
        Proxy: type["Float"],
        instance: float,
        definitions: registry.Definitions,
    ) -> registry.Thing:
        return registry.Thing(schema={"type": "number"}, definitions=definitions)


@registry.register(str)
class String:
    @classmethod
    def get_json_schema(
        Proxy: type["String"],
        instance: str,
        definitions: registry.Definitions,
    ) -> registry.Thing:
        return registry.Thing(schema={"type": "string"}, definitions=definitions)



from collections.abc import Callable, Mapping
from typing import Any, Literal, NamedTuple, TypeVar

from reactivated import types, utils

DefaultWidgetType = {"tsType": 'generated.Types["Widget"]'}
DefaultModelsType = {"type": "null"}

type_registry: dict[str, tuple[Any]] = {}
global_types: dict[str, Any] = {
    "Widget": DefaultWidgetType,
    "models": DefaultModelsType,
}
template_registry: dict[str, tuple[Any]] = {}
interface_registry: dict[str, tuple[Any]] = {}
value_registry: dict[str, tuple[Any, Literal["primitive", "class", "enum"]]] = {}
definitions_registry: dict[Any, Any] = {}
rpc_registry: types.RPCRegistry = {}

PROXIES = utils.ClassLookupDict({})


Override = TypeVar("Override")


def register(proxied: type[object]) -> Callable[[Override], Override]:
    def inner(proxy: Override) -> Override:
        PROXIES[proxied] = proxy
        return proxy

    return inner


PropertySchema = Mapping[str, Any]


Schema = Mapping[Any, Any]


Definitions = Mapping[str, Schema]


JSON = Any


JSONSchema = Any


class Thing(NamedTuple):
    schema: Schema
    definitions: Definitions

    def dereference(self) -> Schema:
        ref = self.schema.get("$ref")

        # Should probably error or Thing should be a subclass for cached
        # schemas that has this property
        if not ref:
            return self.schema

        return self.definitions[ref.replace("#/$defs/", "")]

    def add_property(
        self, name: str, property_schema: PropertySchema, *, optional: bool = False
    ) -> "Thing":
        ref: str | None = self.schema.get("$ref")

        if ref is None:
            assert False, "Can only add properties to ref schemas"

        definition_name = ref.replace("#/$defs/", "")
        dereferenced = self.definitions[definition_name]

        # In case we are replacing a property.
        required = (
            dereferenced["required"]
            if (optional is True or name in dereferenced["required"])
            else [*dereferenced["required"], name]
        )

        return Thing(
            schema=self.schema,
            definitions={
                **self.definitions,
                definition_name: {
                    **dereferenced,
                    "properties": {
                        **dereferenced["properties"],
                        name: property_schema,
                    },
                    "required": required,
                    "additionalProperties": False,
                },
            },
        )
